#!/usr/bin/env python3
import os,sys
paths = os.environ.get('PATH')
paths = paths.split(':')
for path in paths:
	if 'scripts' in path:
		sys.path.append(path)
		break
from Utils import *
import numpy as np
import matplotlib
import matplotlib.pyplot as plt
import argparse
from pyquaternion import Quaternion
from scipy.optimize import curve_fit
matplotlib.rc('text', usetex=True)
matplotlib.rc('text.latex', preamble=r'\usepackage{amsmath}')

#~~~~~~~~~~
#  Parser
#~~~~~~~~~~
parser = argparse.ArgumentParser()
parser.add_argument('--t',type=StrToNumpy,default='0:800:1',help='time')
parser.add_argument('--sides',type=int,default=5,help='number of sides')
parser.add_argument('--ecs',type=str,default='0.8,1.6,2.4',help='ecs')
args = parser.parse_args()

#~~~~~~~~~~
#  Arrays
#~~~~~~~~~~
times = args.t
ecs = np.fromstring(args.ecs,dtype=float,sep=',')
folders = ["./ec{:.1f}".format(ec) for ec in ecs]
alphass = np.empty((len(folders),times.size))
for fi,folder in enumerate(folders):
	for ti,time in enumerate(times):
		_,_,_,_,_,_,q = GetSide(where=folder,time=time)
		alphass[fi,ti] = q.degrees

#~~~~~~~~~
# new axe
#~~~~~~~~~
fig = plt.figure(figsize=(5,5))
ax = fig.add_axes([0.1,0.1,0.6,0.4])
fontSize = 10
frameWidth = 0.5
xticks = np.linspace(0,600,num=7)
xticklabels = [r'${:d}$'.format(int(x)) for x in xticks]
yticks =  np.arange(0,121,30)
yticklabels = [r'${:d}^\circ$'.format(int(y)) for y in yticks]
ax.set_xlim(xticks[0],xticks[-1])
ax.set_ylim(yticks[0],yticks[-1])
ax.set_xticks(xticks)
ax.set_yticks(yticks)
ax.set_xticklabels(xticklabels)
ax.set_yticklabels(yticklabels)
ax.set_xlabel(r'$t/t_0$',fontsize=10,labelpad=4)
ax.set_ylabel(r'$\alpha$',fontsize=10,labelpad=2)
ax.xaxis.set_tick_params(which='major',length=4,direction='in',width=frameWidth,pad=5)
ax.yaxis.set_tick_params(which='major',length=4,direction='in',width=frameWidth,pad=5)
ax.xaxis.set_tick_params(which='minor',length=2,direction='in',width=frameWidth,pad=5)
ax.yaxis.set_tick_params(which='minor',length=2,direction='in',width=frameWidth,pad=5)
ax.xaxis.set_ticks_position('both')
ax.yaxis.set_ticks_position('both')
for axis in ['top','bottom','left','right']:
	ax.spines[axis].set_linewidth(frameWidth)
for item in ([ax.title, ax.xaxis.label, ax.yaxis.label] + ax.get_xticklabels() + ax.get_yticklabels()):
	item.set_fontsize(fontSize)

#~~~~~~~~~~~
# Plot data
#~~~~~~~~~~~
#https://colorbrewer2.o
colors=['#bdd7e7','#6baed6','#3182bd','#08519c']
for i,alphas in enumerate(alphass):
	ax.plot(times,
	        alphas,
	        #marker = 'o',
	        color =colors[i%(len(colors))],
	        label=r'$Ec = {}$'.format(ecs[i]))
	print("  alpha(t->infty = {:.1f}".format(alphas[-1]))
	
#~~~~~~~~~~~~~
# Plot images
#~~~~~~~~~~~~~
img0 = plt.imread("fig_a.png")
img1 = plt.imread("fig_b.png")
img2 = plt.imread("fig_c.png")
for img,alphas in zip([img0,img1,img2],alphass):
	L = 0.15
	bb = ax.get_position()
	x = 445
	y = alphas[-1] + 8
	x,y = TransformAxToFig(ax,x,y)
	ax_tmp = fig.add_axes([x,y-L/2,L,L])
	ax_tmp.imshow(img)
	ax_tmp.axis('off')

#~~~~~~~~
# Legend
#~~~~~~~~
ax.legend()
handles, labels = ax.get_legend_handles_labels()
sort=[2,1,0]
labels = [labels[i] for i in sort]
handles = [handles[i] for i in sort]
ax.legend(handles,
	      labels,
	      loc='upper left',
	      frameon = False,
	      handletextpad = 0.2,
	      handlelength = 0.5)

fig.savefig('fig.pdf',dpi=400)
plt.close(fig)

#~~~~~~~~~
# 3D Drop
#~~~~~~~~~
for ec in ecs:
	script = "load" + str(ec) + '.py'
	ParaviewScript(where='./ec'+str(ec),sides=args.sides,time=times[-1],script=script)

#~~~~~
# tmp
#~~~~~
#There is a jump in the force for ec = 1.2, it is because the tpcl is increased by two additional segment near the hinge
#index=-1
#alphas=np.empty(times.size)
#forces=np.empty((times.size,3))
#for ti,time in enumerate(times):
	#forces[ti],_ = GetNormalizedCapillaryForce(where=folders[index],time=time,ec=ecs[index],info=False)
	#alphas[ti] = alphass[index][ti]
#np.savetxt('force.txt',forces,fmt='%.8f')
#np.savetxt('alpha.txt',alphas,fmt='%.1f')
#[[alpha,F[0],F[1],F[2]] for alpha,F in zip(alphass[fi],forcess[fi])]
#gnuplot -p -e "f='ec1.2/force.txt'; plot f u 1:7, f u 1:9"
#gnuplot; plot 'force.txt' u 1 w p, 'force.txt' u 3 w p, 'ec0.8/force.txt' u ($1/50):7 w l, 'ec0.8/force.txt' u ($1/50):9 w l
#gnuplot; func(x,y)=sqrt(x*x+y*y); f='force.txt'; a='alpha.txt'; plot f u 1 ti 'fx' axis x1y1, f u 3 ti 'fz', f u (func($1,$3)) w lp ti '|f|', a w lp axis x1y2
