#!/usr/bin/env bash
source Utils.sh
finput_="sim.in"
thetas_=($(StrToNumpy "20:141:10"))
nxs_=("100" "150" "200")
submit_=false
function Main()
{
	function PrintUsage()
	{
		echo "  Run simulations related to drop in equilibrium on a flat plate (Figure 7)"
		echo "  --help   help"
		echo "  --theta  contact angle (from:to:step)"
		echo "  --nx     grid size (from:to:step)"
		echo "  --submit submit job using sbatch"
	}
	local longs='help,theta:,nx:,submit'
	options=$(getopt --options '' --long "${longs}" --name "$0" -- "$@")
	if [[ $? != 0 ]]
	then
		PrintUsage
		return
	fi
	eval set -- "${options}"
	while true
	do
		case "$1" in
			--help) PrintUsage; return;;
			--theta) local str="$2"; thetas_=($(StrToNumpy ${str})); shift 2;;
			--nx) local str="$2"; nxs_=($(StrToNumpy ${str})); shift 2;;
			--submit) submit_=true; shift;;
			--) shift ; break;;
			 *) echo "Failed parsing option $1"; PrintUsage; return;;
		esac
	done
	for ((i=0;i<${#thetas_[@]};i++))
	do
		local theta=${thetas_[i]}
		thetas_[i]=$(printf "%03d" ${theta})
	done
	unset options
	echo "  theta = ${thetas_[*]}"
	echo "  nx = ${nxs_[*]}"
	echo "  submit = ${submit_}"
	for nx in ${nxs_[@]}
	do
		nx_num=$(expr ${nx} + 0)
		ny_num=$(expr ${nx} + 0)
		nz_num=$(expr ${nx} / 2)
		for theta in ${thetas_[@]}
		do
			theta_num="$(expr ${theta} + 0)"
			wdir=$(pwd)
			odir="${wdir}/nx${nx}/theta${theta}"
			mkdir -pv ${odir}
			cp ${finput_} ${odir}
			UpdateInputFile --in "${odir}/${finput_}" --key "contact_angle" --val ${theta_num}
			UpdateInputFile --in "${odir}/${finput_}" --key "nx" --val ${nx_num}
			UpdateInputFile --in "${odir}/${finput_}" --key "ny" --val ${ny_num}
			UpdateInputFile --in "${odir}/${finput_}" --key "nz" --val ${nz_num}
			Hemera --in ${finput_} --out ${odir}/hemera.sh --jobname "nx${nx_num}ca${theta_num}" --time 6 --np 8
			echo "source Utils.sh; ExportData --verbose" >> ${odir}/hemera.sh
			if ${submit_}; then cd ${odir}; sbatch hemera.sh; cd ${wdir}; fi
		done
	done
}
Main "$@"
