#!/usr/bin/env python3
import os,sys
paths = os.environ.get('PATH')
paths = paths.split(':')
for path in paths:
	if 'scripts' in path:
		sys.path.append(path)
		break
from Utils import *
import numpy as np
from pyquaternion import Quaternion
import matplotlib.pyplot as plt
from matplotlib.patches import Circle
from matplotlib.collections import PatchCollection
import matplotlib
import subprocess
matplotlib.rc('text', usetex=True)

def ResamplePolyline(xy,n=100):
    d = np.cumsum(np.r_[0, np.sqrt((np.diff(xy, axis=0) ** 2).sum(axis=1))])
    d_sampled = np.linspace(0, d.max(), n)
    xy_interp = np.c_[np.interp(d_sampled,d,xy[:, 0]),np.interp(d_sampled, d, xy[:, 1]),]
    return xy_interp

def dist(polylinea,polylineb,n=-1,method=0):
	pa = polylinea
	pb = polylineb
	if n>0:
		pa = ResamplePolyline(polylinea,n)
		pb = ResamplePolyline(polylineb,n)
	u=pa[1]-pa[0]; u=u/np.linalg.norm(u)
	v=pb[1]-pb[0]; v=v/np.linalg.norm(v)
	if u[0]*v[0]<0 or u[1]*v[1]<0:
		#print("ordering does not match")
		pb = np.flip(pb,axis=0)
	#print("  pa =",pa)
	#print("  pb =",pb)
	dists = np.zeros(len(pa))
	if method==0:
		for i,Ai in enumerate(pa):
			#disti = np.sqrt([(Ai[0]-Bi[0])**2 + (Ai[1]-Bi[1])**2 for Bi in pb])
			disti = np.linalg.norm(pb-Ai,axis=1)
			dists[i] = disti.min()
	if method==1:
		#dists = np.sqrt([(A[0]-B[0])**2 + (A[1]-B[1])**2 for A,B in zip(pa,pb)])
		dists = np.linalg.norm(pa-pb,axis=1)
	return np.mean(dists) #np.median is also possible

#~~~~~~
#  Ax
#~~~~~~
fontSize = 12
frameWidth = 0.5
fig = plt.figure(figsize=(5,5))
ax = fig.add_axes([0.15,0.15,0.4,0.4])
xticks = np.arange(4,17)/10.
yticks = np.arange(2,7)/100.
xticklabels = [r'${}$'.format(x) if x in [0.5,1.0,1.5] else '' for x in xticks]
yticklabels = [r'${}$'.format(y) for y in yticks]
ax.set_xlim(xticks[0],xticks[-1])
ax.set_ylim(yticks[0],yticks[-1])
ax.set_xticks(xticks)
ax.set_yticks(yticks)
ax.set_xticklabels(xticklabels)
ax.set_yticklabels(yticklabels)
#ax.set_xlabel(r'$V_d/\mathcal{V}$',fontsize=10,labelpad=4)
ax.text(1.4,0.0215, r'$V_d/\mathcal{V}$',fontsize=10)
ax.set_ylabel(r'$\Delta/s$',fontsize=10,labelpad=4)
ax.xaxis.set_tick_params(which='major',length=2,direction='in',width=frameWidth,pad=5)
ax.yaxis.set_tick_params(which='major',length=2,direction='in',width=frameWidth,pad=5)
#ax.xaxis.set_tick_params(which='minor',length=2,direction='in',width=frameWidth,pad=5)
#ax.yaxis.set_tick_params(which='minor',length=2,direction='in',width=frameWidth,pad=5)
ax.xaxis.set_ticks_position('both')
ax.yaxis.set_ticks_position('both')
for axis in ['top','bottom','left','right']:
	ax.spines[axis].set_linewidth(frameWidth)
for item in ([ax.title, ax.xaxis.label, ax.yaxis.label] + ax.get_xticklabels() + ax.get_yticklabels()):
	item.set_fontsize(fontSize)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Read side triangle without transform
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
A,B,C,_,_,centre,q = GetSide(where='V1000',time=1,transform=False)
xs_side = [A[1],B[1],C[1]]; xs_side = np.hstack((xs_side,xs_side[0]));
ys_side = [A[0],B[0],C[0]]; ys_side = np.hstack((ys_side,ys_side[0]));
R = (q.inverse).rotation_matrix
side = np.linalg.norm(B-A)
height = np.linalg.norm(C-(A+B)/2)
n = 4
base_area=(1/4)*n*(side**2)/np.tan(np.pi/n)
volume_pyramide=(1/3)*base_area*height
print('  side =',side)
print('  height =',height)
print("  base area = {:.2f}".format(base_area))
print("  pyramide height = {:.2f}".format(height))
print("  pyramide volume = {:.2f}".format(volume_pyramide))
#fig, ax = plt.subplots()
#ax.set_aspect('equal')
#ax.plot(xs_side,ys_side,color='black',linewidth=0.5)

#~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Select volumes to plot
#~~~~~~~~~~~~~~~~~~~~~~~~~~
plotV = np.array([2000,3600,5400])
Vi = plotV[0]/volume_pyramide
Vj = plotV[1]/volume_pyramide
Vk = plotV[2]/volume_pyramide
print('  plot tpcl for V_d/V = {:.2f}, {:.2f}, {:.2f}'.format(Vi,Vj,Vk,))
xs_sim,ys_sim=[],[]
xs_th,ys_th=[],[]

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Plot distance error against volume
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
theta = 70
volumes = np.arange(1800,6001,200)
distances = np.empty(volumes.shape)
for i,volumei in enumerate(volumes):
	#Simulation
	print("  Importing simulated tpcl")
	polylines = ImportPolylines(where='./V' + str(volumei),name='tpcl00',time=1,info=True)
	for polyline in polylines:
		if polyline.size>0:
			polyline = np.array([R.dot(pt-centre) for pt in polyline])
			polyline = ResamplePolyline(polyline)
			pa = polyline
			if volumei in plotV:
				xs = pa[:,1]; xs_sim.append(xs)
				ys = pa[:,0]; ys_sim.append(ys)

	#Analytical
	print("  Importing semi-analytical tpcl")
	p = subprocess.run('python3 Sphere.py --theta {} --volumes {} --silent yes'.format(theta,volumei),shell=True)
	polylines = ImportPolylines(where='.',name='tpcl',time=-1,cloudsize=1,info=True)
	for polyline in polylines:
		if polyline.size>0:
			polyline = np.array([R.dot(row-centre) for row in polyline])
			polyline = ResamplePolyline(polyline)
			pb = polyline
			if volumei in plotV:
				xs = pb[:,1]; xs_th.append(xs);
				ys = pb[:,0]; ys_th.append(ys);
	distances[i] = dist(pa,pb)
ax.plot(volumes/volume_pyramide,
        distances/side,
        marker = 'o',
        markersize = 5,
        markerfacecolor = 'white',
        markeredgecolor = 'C0',
        markeredgewidth = 1,
        linewidth = 0)

#~~~~~~~~~~~~~~~~~
#  Select points
#~~~~~~~~~~~~~~~~~
indices = np.argwhere(np.isin(volumes,plotV)).flatten()
xs = np.take(volumes,indices)/volume_pyramide
ys = np.take(distances,indices)/side
ax.plot(xs,
	    ys,
	    marker = 'o',
	    markersize = 4,
	    markerfacecolor = 'C0',
	    markeredgecolor = 'None',
	    markeredgewidth = 0,
	    linewidth = 0)
labels=['i','j','k']
for label,x,y in zip (labels,xs,ys):
	ax.text(x-0.04,y+0.0005,r'${}$'.format(label),fontsize=fontSize,ha='center',va='bottom',color='black')

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Create new axes + plot triangle side
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
x0 = ax.get_xlim()[1] * 1.05
y0 = ax.get_ylim()[0]
dx = 0.35
dy = -1
height = (ax.get_ylim()[1] - ax.get_ylim()[0])*0.8
x0_all = [x0,x0+dx,x0+2*dx]
y0_all = [y0,y0,y0]
alpha = (xs_side.max()-xs_side.min())/(ys_side.max()-ys_side.min()) #aspect ratio
_,h0 = TransformAxToFig(ax,-1,y0       )
_,h1 = TransformAxToFig(ax,-1,y0+height)
ly = h1-h0
lx = alpha*ly
axs = []

for i,(x0,y0) in enumerate(zip(x0_all,y0_all)):
	bb = ax.get_position()
	x0,y0 = TransformAxToFig(ax,x0,y0)
	axs.append(fig.add_axes([x0,y0,lx,ly]))
	axs[-1].axis('off')
	axs[-1].plot(xs_side,ys_side,color='black',linewidth=0.5)
	axs[-1].set_xlim([xs_side.min(),xs_side.max()])
	axs[-1].set_ylim([ys_side.min(),ys_side.max()])
	axs[-1].set_xticks([])
	axs[-1].set_yticks([])
	axs[-1].text(x0+lx/2,y0+ly/2,r'${}$'.format(labels[i]),fontsize=fontSize,ha='center',va='center',color='black',transform=fig.transFigure)
	if i<len(xs_th):
		xs,ys = xs_th[i] ,ys_th[i] ; axs[-1].plot(xs,ys,color='C1',linewidth=2.0,label='Analytical')
		xs,ys = xs_sim[i],ys_sim[i]; axs[-1].plot(xs,ys,color='C0',linewidth=2.0,label='Simulation')
Arrow(axs[0],[0.0,-0.05],[1,-0.05],arrowstyle='<|-|>',xycoords='axes fraction',head_length=0.2,head_width=0.1)
TextBox(axs[0],[0.5,-0.05],r'$s$',linewidth=0,pad=1,xycoords='axes fraction',fontsize=5)

bb = axs[1].get_position()
x_legend = (bb.x0 + bb.x1)/2
y_legend = bb.y1*0.975
axs[0].legend(loc = 'lower center',
              bbox_to_anchor = (x_legend,y_legend),
              bbox_transform = fig.transFigure,
              frameon = False)
fig.savefig('fig.pdf')
plt.close(fig)
